module BranchableCDNAssets

  # this abstracts creating filter methods
  # not offering inheritance currently
  #
  #     check_before :check, params={}
  #
  # to only check for specific methods,
  # pass an array of methods to the :methods key
  #
  #     check_before :check, methods: ["foo", "bar"]
  #
  module CheckBefore

    def before_checks
      @_before_checks ||= []
    end

    # define a before check for a specific method
    #
    # @param method [Symbol] the method to check on
    # @param check [Symbol] the check to run
    # @param params [Hash] additional parameters for the check
    # @return [Void]
    def check_before check, params={}
      before_checks << params.merge( check: check )
    end


    def self.extended base
      base.class_eval do

        # @param action [Symbol] the method to execute
        # @param *args [Array] arguments to pass to the check method
        def with_check action, *args
          self.class.before_checks.select do |c|
            c[:methods].nil? || c[:methods].include?(action)
          end.each do |c|
            send c[:check]
          end

          public_send(action, *args)
        end

      end
    end

  end
end