module BranchableCDNAssets
  module Invalidator
    class AkamaiCCU < Base

      PURGE_PATH  = '/ccu/v2/queues'.freeze
      STATUS_PATH = '/ccu/v2/queues'.freeze
      PURGE_STATUS_PATH = '/ccu/v2/purges'.freeze

      register_as :akamai
      require_keys [:base_url, :access_token, :client_token, :secret_token]

      set_defaults domain: 'production',
                   queue_name: 'default'

      def invalidate_files files
        url  = File.join( config.base_url, PURGE_PATH, config.queue_name )
        resp = HTTP.post( { access_token: config.access_token,
                            client_token: config.client_token,
                            secret_token: config.secret_token
                          }.merge(  url: url,
                                    body: {
                                      action: 'invalidate',
                                      domain: config.domain,
                                      objects: Array(files)
                                    }))


        if resp.code.to_i == 201
          puts "Posted invalidation for #{files.count} files:"
        else
          puts "Invalidation failed:"
        end

        JSON.parse(resp.body).each do |k,v|
          puts "-> #{k}: #{v}"
        end
      end

      # queue status or purge status
      def status id=nil
        if id
          resp = HTTP.get({ access_token: config.access_token,
                            client_token: config.client_token,
                            secret_token: config.secret_token
                          }.merge(  url: File.join( config.base_url, PURGE_STATUS_PATH, id ) ))
        else
          resp =  HTTP.get({ access_token: config.access_token,
                             client_token: config.client_token,
                             secret_token: config.secret_token
                           }.merge(  url: File.join( config.base_url, STATUS_PATH, config.queue_name ) ))
        end

        puts "Status:"
        JSON.parse(resp.body).each do |k,v|
          puts "-> #{k}: #{v}"
        end
      end

    end
  end
end

require_relative 'akamai_ccu/http'
