module BranchableCDNAssets
  module Invalidator
    class Base

      class << self
        def register_as key
          BranchableCDNAssets::Invalidator.register key, self
        end

        def require_keys keys
          @required_keys = Array(keys)
        end

        def required_keys
          @required_keys ||= []
        end

        def set_defaults opts={}
          @defaults = opts
        end

        def defaults
          @defaults ||= {}
        end
      end

      attr_reader :config

      def initialize config_opts={}
        config_opts = self.class.defaults.merge config_opts
        unless (self.class.required_keys - config_opts.keys).empty?
          raise ArgumentError, "Missing required config keys: [#{(self.class.required_keys - config_opts.keys).join(', ')}]"
        end

        @config = Struct.new("Config_#{object_id}", *config_opts.keys).new(*config_opts.values)
      end

      # @param files [Array]
      def invalidate_files files
        raise NoMethodError, "invalidate_files hasn't been implemented for #{self.class}"
      end

    end
  end
end
