require 'set'

module BranchableCDNAssets
  class Manifest

    attr_reader :source_file, :file_set

    def initialize source_file
      @source_file = source_file
      @file_set    = File.exists?(source_file) ? read_source_file : Set.new
    end

    # list files set as an Array
    # @return [Array]
    def files
      file_set.to_a
    end

    # add new files to the manifest set
    # @param new_files [Array]
    # @return [Set]
    def merge_files new_files
      file_set.merge Array(new_files)
    end

    # remove a set of files from the manifest
    # @param files_to_remove [Array]
    # @return [Set]
    def remove_files files_to_remove
      file_set.subtract Array(files_to_remove)
    end

    # updates the manifest file with the
    # most recent set of files
    # @return [Void]
    def update_source_file!
      destroy! && return if file_set.empty?

      File.open( source_file, 'w' ) do |file|
        file.write manifest_content
      end
    end

    private

      # read the manifest source_file
      def read_source_file
        IO.read(source_file).split.to_set
      end

      # render/serialize the manifest set
      def manifest_content
        file_set.to_a.join("\n")
      end

      # remove the manifest source_file
      def destroy!
        File.delete(source_file) if File.exist?(source_file)
        true
      end

  end
end
