require 'rake'
require 'branchable_cdn_assets'

module BranchableCDNAssets

  class RakeTasks
    include ::Rake::DSL

    class << self
      def register namespace, data={}
        RakeTasks.new( namespace, Config.new(data) ).register_tasks
      end
    end

    attr_reader :file_manager, :rake_namespace

    def initialize namespace, config
      @file_manager   = FileManager.new config
      @rake_namespace = namespace
    end

    def tasks
      {
        list:   'list of local files',
        pull!:  'move the current branch\'s remote files to local',
        push!:  'move local files to the current branch\'s remote',
        prune!: 'remove local files with the same name as remote files',
        move_to_production: 'move named branch files to production cdn',
        doctor: 'list file differences between remotes',
        heal:   'pulls then pushes necessary files to make all remotes lists match',
        setup:  'setup dir on remotes'
      }
    end

    def register_tasks
      tasks.each do |name, desc|
        if self.respond_to?(:"register_#{name}")
          self.send(:"register_#{name}", desc)
        else
          register_task name, desc
        end
      end
    end

    def register_move_to_production task_desc
      in_namespace do
        desc task_desc
        task :move_to_production, :branch do |t,args|
          puts file_manager.with_check(:move_to_production, args[:branch])
        end
      end
    end

    private

      def in_namespace &block
        namespace rake_namespace do
          yield
        end
      end

      def register_task task_name, task_desc
        in_namespace do
          desc task_desc
          task task_name.to_s.sub('!', '') do
            puts file_manager.public_send( :with_check, task_name )
          end
        end
      end

  end
end